package de.rmdir.ms2debounce;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.DataOutputStream;

import android.content.Context;
import android.content.SharedPreferences;

public class DebounceModuleHelper
{
	private Context ctx;
	public static final String PREFS_NAME = "DebounceCfg";
	final int SUPERUSER_REQUEST = 4223;

	public DebounceModuleHelper(Context context) {
		ctx = context;
	}

	public void setAllValues() {
		setDelay(getSavedDelay());
		setSettle(getSavedSettle());
		setPoll(getSavedPoll());
	}

	public void loadModule() {
		_loadModule();
		setAllValues();
	}

	protected void runAsRoot(String command) throws java.io.IOException,java.lang.InterruptedException {
		Process rootcmd = Runtime.getRuntime().exec(new String[]{"su","-c","sh"});
		DataOutputStream sh = new DataOutputStream(rootcmd.getOutputStream());
		sh.writeBytes(command + "\n");
		sh.writeBytes("exit\n");
		sh.flush();
		sh.close();

		rootcmd.waitFor();
	}

	public synchronized void _loadModule() {
		File debounce_ko = new File(ctx.getFilesDir() + "/debounce.ko");

		extractModule();

		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();
		if (is_safe_to_load()) {
			editor.putBoolean("safe_to_load", false);
			editor.commit();
		}

		try {
			runAsRoot("/system/bin/insmod " + debounce_ko);
		} catch (Exception e) {
			return;
		}

		if (!isLoaded()) {
			return;
		}

		if (getDelay() < 0) {
			return;
		}

		/* Module was obviously loaded, so it is safe to load on boot */
		editor.putBoolean("safe_to_load", true);
		editor.commit();
	}

	public synchronized void unloadModule() {
		try {
			runAsRoot("/system/bin/rmmod debounce");
		} catch (Exception e) {}
	}

	public synchronized boolean isLoaded() {
		boolean loaded = false;
		try {
			String read;

			FileReader modules = new FileReader("/proc/modules");
			BufferedReader modules_buf = new BufferedReader(modules);

			while((read = modules_buf.readLine()) != null) {
				if (read.regionMatches(0, "debounce", 0, 8)) {
					loaded = true;
				}
			}

		} catch (Exception e) {
			loaded = false;
		}

		return loaded;
	}

	private synchronized int getValue(String parameter) {
		int value = -1;

		try {
			String read;

			FileReader fr = new FileReader("/sys/devices/debounce/" + parameter);
			BufferedReader fbuf = new BufferedReader(fr);

			read = fbuf.readLine();
			if (read != null) {
				value = Integer.parseInt(read.trim());
			}

			fbuf.close();
		} catch (Exception e) {}

		return value;
	}

	private synchronized void setValue(String parameter, int value) {
		if (!isLoaded()) {
			return;
		}

		try {
			FileWriter fw = new FileWriter("/sys/devices/debounce/" + parameter);
			BufferedWriter fbuf = new BufferedWriter(fw);

			fbuf.write((new Integer(value)).toString());

			fbuf.close();
		} catch (Exception e) {}
	}

	public synchronized int getDelay() {
		return getValue("debounce_delay");
	}

	public synchronized void setDelay(int debounce_delay) {
		setValue("debounce_delay", debounce_delay);
	}

	public synchronized int getSettle() {
		return getValue("settle_time");
	}

	public synchronized void setSettle(int settle_time) {
		setValue("settle_time", settle_time);
	}

	public synchronized int getPoll() {
		return getValue("poll_time");
	}

	public synchronized void setPoll(int poll_time) {
		setValue("poll_time", poll_time);
	}

	public synchronized int getSavedDelay() {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);

		return settings.getInt("debounce_delay", 15);
	}

	public synchronized void setSavedDelay(int delay) {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();

		editor.putInt("debounce_delay", delay);
		editor.commit();
	}

	public synchronized int getSavedSettle() {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);

		return settings.getInt("settle_time", 150);
	}

	public synchronized void setSavedSettle(int settle) {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();

		editor.putInt("settle_time", settle);
		editor.commit();
	}

	public synchronized int getSavedPoll() {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);

		return settings.getInt("poll_time", 2);
	}

	public synchronized void setSavedPoll(int poll) {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();

		editor.putInt("poll_time", poll);
		editor.commit();
	}

	public synchronized boolean is_safe_to_load() {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		return settings.getBoolean("safe_to_load", false);
	}

	public synchronized boolean get_on_boot() {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		return settings.getBoolean("on_boot", false);
	}

	public synchronized void set_on_boot(boolean on_boot) {
		SharedPreferences settings = ctx.getSharedPreferences(PREFS_NAME, Context.MODE_PRIVATE);
		SharedPreferences.Editor editor = settings.edit();

		editor.putBoolean("on_boot", on_boot);
		editor.commit();
	}

	private synchronized void extractModule() {
		File debounce_ko = new File(ctx.getFilesDir() + "/debounce.ko");

		if (debounce_ko.exists()) {
			return;
		}

		try {
			InputStream apk = ctx.getAssets().open("debounce.ko");
			OutputStream mod = ctx.openFileOutput("debounce.ko.tmp", 0);

			//I assume a page is 4k...
			byte buf[] = new byte[4096];
			int bytes;

			while((bytes = apk.read(buf)) != -1) {
				mod.write(buf, 0, bytes);
			}

			apk.close();
			mod.close();

			File tmpfile = new File(debounce_ko + ".tmp");
			tmpfile.renameTo(debounce_ko);
		} catch (Exception e) {}
	}
}
